#include "wios.h"
#include "lsf.h"

static const sf_pcm_format_t sf_format[] = {
	SF_FORMAT_PCM_U8,
	SF_FORMAT_PCM_16,
	SF_FORMAT_PCM_24,
	SF_FORMAT_PCM_32,
	SF_FORMAT_UNKNOWN
};



/* GLOBAL FUNCTION */
/* -------------------------------------------------- */

/* フォーマットが扱えるかを調べて対応するフォーマットを返す */
wios_pcm_format_t lsf_determin_format(sf_pcm_format_t sf_fmt)
{
	int i, n = ARRAY_SIZE(sf_format);
  int type = sf_fmt & SF_FORMAT_TYPEMASK;
  int sub  = sf_fmt & SF_FORMAT_SUBMASK;

  /* MSWAVとMSWAVXのみ対応 */
  if (type == SF_FORMAT_WAV || type == SF_FORMAT_WAVEX) {
		for (i = 0; i < n-1; i++)
			if (sub == sf_format[i]) return i;

		return n-1;
  }
}

void lsf_init_read(lsf_t * sf)
{
	sf->sndfile = NULL;
	sf->sfinfo.format = 0;
}

void lsf_init_write(wios_file_t * ft, lsf_t * sf)
{
	sf->sndfile = NULL;
	/* MSWAVフォーマットのみ対応 */
	sf->sfinfo.format = sf_format[ft->fmtinfo.format] | SF_FORMAT_WAV;
	sf->sfinfo.samplerate = ft->fmtinfo.rate;
	sf->sfinfo.channels = ft->fmtinfo.channels;
	sf->sfinfo.sections = 1;
	sf->sfinfo.seekable = 1;
}

/* 読み取り専用にファイルを開いて構造体初期化 */
wios_file_t * lsf_open_read(wios_file_t * ft) {
	lsf_t * sf = &ft->sf;
	wios_fmtinfo_t * p = &ft->fmtinfo;

	lsf_init_read(sf);
	if ((sf->sndfile = sf_open(ft->file_name, SFM_READ, &sf->sfinfo)) == NULL)
		WIOS_ERROR("cannot open D/A file (%s)", ft->file_name);

	p->rate = sf->sfinfo.samplerate;
	p->channels = sf->sfinfo.channels;
	p->format = lsf_determin_format(sf->sfinfo.format);
	ft->signal.all_frames = sf->sfinfo.frames;

	ft->fp = sf->sndfile;
  ft->signal.buf_size = ft->signal.execinfo.periodsize * p->channels;
	ft->signal.byte_per_item = byte_per_item[p->format];

	if (p->format == WIOS_PCM_FORMAT_UNKNOWN)
		WIOS_ERROR("cannot play this format %s", wios_format_to_string(p->format));

	return ft;

error:
	return NULL;
}

/* 書き込み専用にファイルを開いて構造体初期化 */
wios_file_t * lsf_open_write(wios_file_t * ft) {
	lsf_t * sf = &ft->sf;

	lsf_init_write(ft, sf);
	if (sf_format_check(&sf->sfinfo) == 0)
		WIOS_ERROR("cannot use this format", NULL);
	if ((sf->sndfile = sf_open(ft->file_name, SFM_WRITE, &sf->sfinfo)) == NULL)
		WIOS_ERROR("cannot open A/D file (%s)", ft->file_name);

	ft->fp = sf->sndfile;
	ft->signal.buf_size = ft->signal.execinfo.periodsize * ft->fmtinfo.channels;
	ft->signal.byte_per_item = byte_per_item[ft->fmtinfo.format];

	return ft;

error:
	return NULL;
}

/* ファイルに波形データをnframesサンプルだけbufから書き込む */
int lsf_file_write(wios_file_t * ft, int32_t * buf, int nframes)
{
	int err;

	if ((err = sf_writef_int(ft->fp, buf, nframes)) < 0)
		WIOS_WARN("error in writing file (%s)", sf_strerror(ft->fp));
	
	return err;
}

/* ファイルから波形データをnframesサンプルだけbufに読み込む */
int lsf_file_read(wios_file_t * ft, int32_t * buf, int nframes)
{
	int err;

	if ((err = sf_readf_int(ft->fp, buf, nframes)) != nframes)
		WIOS_WARN("error in reading file (%s)", sf_strerror(ft->fp));

	return err;
}
